/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: ComponentElement.java,v 1.3 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.component;

import javax.jbi.JBIException;
import javax.jbi.component.Bootstrap;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.InstallationContext;
import javax.jbi.management.ComponentLifeCycleMBean;
import javax.jbi.management.InstallerMBean;
import javax.management.ObjectName;

import org.objectweb.petals.component.event.StateChangeFailedEvent;
import org.objectweb.petals.component.state.ElementState;
import org.objectweb.petals.util.ESBLogger;


/**
 * <code>ComponentElement</code> contains all information and reference 
 * related to a JBI component (binding or engine component), such a reference
 * to the component, its context, its installation context,...
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class ComponentElement extends LifeCycleElement implements ComponentLifeCycleMBean, InstallerMBean
{
    private Component component = null;
    
    private Bootstrap bootstrap = null;
    
    private InstallationContext installationContext = null;
    
    private ComponentContext componentContext = null;
    
    private boolean installerInitialized = false;
    
    
    /**
     * Creates a <code>ComponentElement</code>
     * @param component the reference to the component
     * @param context the context of the component
     */
    public ComponentElement(Component component, ComponentContext context)
    {
        this.component 			= component;
        this.componentContext 	= context;
    }
    
    public Bootstrap getBootstrap()
    {
        return bootstrap;
    }
    public Component getComponent()
    {
        return component;
    }
    public ComponentContext getComponentContext()
    {
        return componentContext;
    }
    public InstallationContext getInstallationContext()
    {
        return installationContext;
    }
    
    public String getName()
    {
        return componentContext.getComponentName();
    }
    
    /**
     * @see javax.jbi.management.ComponentLifeCycleMBean#getExtensionMBeanName()
     */
    public ObjectName getExtensionMBeanName()
    {
        return component.getLifeCycle().getExtensionMBeanName();
    }
    
    /**
     * TODO getInstallerConfigurationMBean
     * @see javax.jbi.management.InstallerMBean#getInstallerConfigurationMBean()
     */
    public ObjectName getInstallerConfigurationMBean() throws JBIException
    {
        return null;
    }
    
    /**
     * TODO getInstallRoot
     * @see javax.jbi.management.InstallerMBean#getInstallRoot()
     */
    public String getInstallRoot()
    {
        return null;
    }
    
    /**
     * TODO isInstallerInitialized
     * @see javax.jbi.management.InstallerMBean#isInstallerInitialized()
     * @return
     */
    public boolean isInstallerInitialized()
    {
        return installerInitialized;
    }
    
    public void setInstallerInitialized(boolean installerInitialized)
    {
        this.installerInitialized = installerInitialized;
    }
    
    ////////////////////////////////////////////////////////
    // Implementation of methods called by 
    // the State when changing it
    ////////////////////////////////////////////////////////
    
    /**
     * Install the element by calling <code>onInstall</code> on
     * the <code>Bootstrap</code> of the element.
     * Init the <code>Bootstrap</code> before if necessary.
     */
    public synchronized void installElement() 
    {
        ESBLogger.logFine(this,"installElement()");
        
        setState(ElementState.installing);
        
        try
        {
            if(! isInstallerInitialized())
            {
                getBootstrap().init(getInstallationContext());
                
                setInstallerInitialized(true);
            }
            getBootstrap().onInstall();
            
            setState(ElementState.installed);
        }
        catch (JBIException e)
        {
            setState(ElementState.unknown);
            fireStateChangeFailed(new StateChangeFailedEvent(this,e));
        }    
    }
    
    /**
     * Uninstall the element by calling <code>onUninstall</code> on
     * the <code>Bootstrap</code> of the element.
     * Init the <code>Bootstrap</code> before if necessary.
     */
    public synchronized void uninstallElement() 
    {
        ESBLogger.logFine(this,"uninstallElement()");
        
        setState(ElementState.uninstalling);
        
        try
        {
            if(! isInstallerInitialized())
            {
                getBootstrap().init(getInstallationContext());
                
                setInstallerInitialized(true);
            }
            getBootstrap().onUninstall();
            
            setState(ElementState.empty);
        }
        catch (JBIException e)
        {
            setState(ElementState.unknown);
            fireStateChangeFailed(new StateChangeFailedEvent(this,e));
        }    
    }
    
    /**
     * Start the element by calling <code>start</code> on
     * the <code>LifeCycle</code> of the element.
     * Init the <code>LifeCycle</code> before if necessary.
     */
    public synchronized void startElement() 
    {
        ESBLogger.logFine(this,"startElement()");
        
        setState(ElementState.starting);
        
        try
        {
            if(! isElementInitialized())
            {
                component.getLifeCycle().init(getComponentContext());
                
                setElementInitialized(true);
            }
            component.getLifeCycle().start();
            
            setState(ElementState.started);
        }
        catch (JBIException e)
        {
            setState(ElementState.unknown);
            fireStateChangeFailed(new StateChangeFailedEvent(this,e));
        }    
    }
    
    /**
     * Stop the element by calling <code>stop</code> on
     * the <code>LifeCycle</code> of the element.
     */
    public synchronized void stopElement() 
    {
        ESBLogger.logFine(this,"stopElement()");
        
        setState(ElementState.stopping);
        
        try
        {
            component.getLifeCycle().stop();
            
            setState(ElementState.stopped);
        }
        catch (JBIException e)
        {
            setState(ElementState.unknown);           
            fireStateChangeFailed(new StateChangeFailedEvent(this,e));
        }    
    }
    
    /**
     * Shutdown the element by calling <code>shutDown</code> on
     * the <code>LifeCycle</code> of the element.
     */
    public synchronized void shutdownElement() 
    {
        ESBLogger.logFine(this,"shutdownElement()");
        
        setState(ElementState.shutdowning);
        
        try
        {
            component.getLifeCycle().shutDown();
            
            setState(ElementState.installed);
        }
        catch (JBIException e)
        {
            setState(ElementState.unknown);
            fireStateChangeFailed(new StateChangeFailedEvent(this,e));
        }    
    }
    
}
