/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: LifeCycleElement.java,v 1.3 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.component;

import javax.jbi.JBIException;

import javax.jbi.management.LifeCycleMBean;
import javax.management.ObjectName;

import org.objectweb.petals.component.event.ElementStateListener;
import org.objectweb.petals.component.event.StateChangeFailedEvent;
import org.objectweb.petals.component.event.StateChangedEvent;
import org.objectweb.petals.component.state.ElementState;


/**
 * <code>LifeCycleElement</code> manages the lifecycle state
 * of all JBI elements (binding, engine and system components).
 * All lifecycle action is launch in a separate thread, 
 * with <code>StateChangedEvent</code> notifications.
 * @author Adrien LOUIS - EBM WebSourcing
 */
public abstract class LifeCycleElement implements LifeCycleMBean
{
    protected ElementState state = null;
    
    protected boolean elementInitialized = false;
    
    protected ElementStateListener elementListener = null;
    
    protected ObjectName mBeanName = null;
    
    /**
     * Abstract constructor.
     * state is set to <i>installed</i>
     *
     */
    public LifeCycleElement()
    {
        state = ElementState.installed;
    }
    
    /**
     * Set the <code>ElementStateListener</code> 
     * which will be notified of state changes
     * @param listener
     */
    public void setElementListener(ElementStateListener listener)
    {
        this.elementListener = listener;
    }
    
    /**
     * Return whether the element is initialized
     * @return
     */
    public boolean isElementInitialized()
    {
        return elementInitialized;
    }
    public void setElementInitialized(boolean componentInitialized)
    {
        this.elementInitialized = componentInitialized;
    }
    
    /**
     * Return the <code>State</code> reference of the current state
     * @return
     */
    public ElementState getState()
    {
        return state;
    }
    
    /**
     * Return the name of the current State
     * @see javax.jbi.management.LifeCycleMBean#getCurrentState() 
     */
    public String getCurrentState()
    {
        return state.getName();
    }
    
    /**
     * Set the <code>State</code> reference of the current state.
     * An <code>ElementStateEvent</code> is thrown.
     * @param state
     */
    public synchronized void setState(ElementState state)
    {
        StateChangedEvent event = new StateChangedEvent(this,this.state,state);
        
        fireStateChanged(event);
        
        this.state = state;
    }
    
    /**
     * The specified <code>ElementStateEvent</code> is fire
     * to the listener.
     * @param event
     */
    protected void fireStateChanged(StateChangedEvent event)
    {
        elementListener.stateChanged(event);
    }
    
    /**
     * The specified <code>StateChangeFailedEvent</code> is fire
     * to the listener.
     * @param event
     */
    protected void fireStateChangeFailed(StateChangeFailedEvent event)
    {
        elementListener.stateChangeFailed(event);
    }
    
    /**
     * Return the name of the element, which can be the
     * name of a component, or a system service.
     * @return
     */
    public abstract String getName();
    
    /**
     * Return the name of the element, which is different from the name
     * @return
     */
    public ObjectName getMBeanName()
    {
        return mBeanName;
    }
    
    public void setMBeanName(ObjectName beanName)
    {
        mBeanName = beanName;
    }
    /**
     * Shutdown the JBI element
     * @see javax.jbi.management.LifeCycleMBean#shutDown()
     */
    public void shutDown() throws JBIException
    {
        state.shutdown(this);
    }
    
    /**
     * Start the JBI element
     * @see javax.jbi.management.LifeCycleMBean#start()
     */
    public void start() throws JBIException
    {
        state.start(this);
    }
    
    /**
     * Stop the JBI element
     * @see javax.jbi.management.LifeCycleMBean#stop()
     */
    public void stop() throws JBIException
    {
        state.stop(this);
    }
    
    /**
     * Install the JBI element
     * @see javax.jbi.management.InstallerMBean#install()
     * TODO install: return parameter
     */
    public ObjectName install() throws JBIException
    {
        state.install(this);
        return null;
    }
    
    /**
     * Return true if the current state is <i>installed</i>
     * @see javax.jbi.management.InstallerMBean#isInstalled()
     */
    public boolean isInstalled()
    {
        return state == ElementState.installed;
    }
    
    /**
     * Uninstall the JBI element
     * @see javax.jbi.management.InstallerMBean#uninstall()
     */
    public void uninstall() throws JBIException
    {
        state.uninstall(this);
    }
    
    /**
     * Start the JBI element in the current thread.
     * This method force the JBI container to wait for 
     * the completion of the starting state before it continues.
     * @throws JBIException
     */
    public void startSync() throws JBIException
    {
        startElement();
    }
    
    
    ////////////////////////////////////////////////////////
    // Actions called by the State when changing it
    ////////////////////////////////////////////////////////
    
    public abstract void installElement(); 
    
    public abstract void uninstallElement(); 
    
    public abstract void startElement(); 
    
    public abstract void stopElement(); 
    
    public abstract void shutdownElement(); 
    
}    
