/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: ConfigHelper.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.config;


import java.net.URL;
import java.util.HashMap;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * <code>ConfigHelper</code> is used to create a <code>ContainerConfig</code>
 * object from a configuration file.
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class ConfigHelper {
    
    /**
     * Return a <i>ContainerConfig</i> object constructs from
     * the file <i>url</i>
     * @param url config-file URL.
     * @return a ServiceContainerConfig
     * @throws Exception
     */
    public ContainerConfig createConfigFromFile(URL url) throws Exception
    {
        Source source = loadConfigurationFile(url);
        
        DOMResult domConfig = makeConfigurationAsDOM(source);
        
        return processConfig(domConfig);
    }
    
    /**
     * Get the <code>Source</code> from the URL
     * @param url
     * @return
     * @throws Exception
     */
    private Source loadConfigurationFile(URL url) throws Exception
    {
        StreamSource source = new StreamSource();
        
        source.setInputStream(url.openStream());
        
        return source;
    }
    
    /**
     * Create a DOM representation of the <code>Source</code>
     * @param source
     * @return
     * @throws Exception
     */
    private DOMResult makeConfigurationAsDOM(Source source) throws Exception
    {
        if(source == null) return null;
        
        DOMResult config = null; 
        
        Transformer transformer = TransformerFactory.newInstance().newTransformer();
        
        config = new DOMResult();
        
        transformer.transform(source, config);
        
        return config;
    }
    
    /**
     * Analysis the DOM representation of the configuration file
     * and constructs the <code>ContainerConfig</code> object.
     * @param domResult
     * @return
     * @throws Exception
     */
    private ContainerConfig processConfig(DOMResult domResult) throws Exception
    {
        ContainerConfig config = new ContainerConfig();
        
        if(domResult == null) return null;
        
        // <JBI>
        Node jbiNode = domResult.getNode().getFirstChild();
        
        if(! "jbi".equals(jbiNode.getNodeName().toLowerCase()))
            throw new Exception("jbi element required");
        
        // <CONTAINER>
        
        Node containerNode = getFirstChild(jbiNode);
        
        if(! "container".equals(containerNode.getNodeName().toLowerCase()))
            throw new Exception("container element required");
        
        // container host
        
        String host = getAttributeValue(containerNode,"host");
        
        if(host == null)
            throw new Exception("parameter 'host' is required for 'container' element");
        
        config.setHost(host);
        
        // container identification
        
        String identification = getAttributeValue(containerNode,"identification");
        
        if(identification == null)
            throw new Exception("parameter 'identification' is required for 'container' element");
        
        config.setIdentification(identification);		
       
        // container adminport
        
        String s_adminPort = getAttributeValue(containerNode,"admin-port");
        
        if(s_adminPort == null)
            throw new Exception("parameter 'admin-port' is required for 'container' element");
        int adminPort = 0;
        try
        {
            adminPort = Integer.parseInt(s_adminPort);
        }
        catch (NumberFormatException e)
        {
            throw new Exception("parameter 'admin-port' must be an Int value");
        }
        
        config.setAdminPort(adminPort);		
       
        // <SYSTEM SERVICES>
        Node systemNode = getNextSibling(containerNode);
        
        if(! "system-services".equals(systemNode.getNodeName().toLowerCase()))
            throw new Exception("system-services element required");
        
        // <DIRECTORY>
        
        Node directoryNode = getFirstChild(systemNode);
        
        if(! "directory".equals(directoryNode.getNodeName().toLowerCase()))
            throw new Exception("directory element required");
        
        config.setDirectory(createSystemServiceConfig(directoryNode));
        
        // <TRANSPORTER>
        
        Node transporterNode = getNextSibling(directoryNode);
        
        if(! "transporter".equals(transporterNode.getNodeName().toLowerCase()))
            throw new Exception("transporter element required");
        
        config.setTransporter(createSystemServiceConfig(transporterNode));
        
        // <ITINERAYRESOLVER>
        
        Node itineraryNode = getNextSibling(transporterNode);
        
        if(! "itinerary-resolver".equals(itineraryNode.getNodeName().toLowerCase()))
            throw new Exception("itinerary-resolver element required");
        
        config.setItineraryResolver(createSystemServiceConfig(itineraryNode));
        
        // <ADDRESSRESOLVER>
        
        Node addressNode = getNextSibling(itineraryNode);
        
        if(! "address-resolver".equals(addressNode.getNodeName().toLowerCase()))
            throw new Exception("address-resolver element required");
        
        config.setAddressResolver(createSystemServiceConfig(addressNode));

        
	    // <COMPONENTS>
        
        Node componentsNode = getNextSibling(systemNode);
        
        if(! "components".equals(componentsNode.getNodeName().toLowerCase()))
            throw new Exception("components element required");
        
        NodeList components = componentsNode.getChildNodes();
        
        for(int i = 0; i<components.getLength(); i++)
        {
            // <COMPONENT>
            Node componentNode = components.item(i);
            
            if("component".equals(componentNode.getNodeName()))
            {
                ComponentConfig component = new ComponentConfig();
                
                // identification attribute
                
                String cidentification = getAttributeValue(componentNode,"identification");
                
                if(cidentification == null)
                    throw new Exception("parameter 'identification' is required for 'component' element");
                
                component.setIdentification(cidentification);
                
                // type attribute
                
                String type = getAttributeValue(componentNode,"type");
                
                if(type == null || (!type.equals(ComponentConfig.BINDING_TYPE) && !type.equals(ComponentConfig.ENGINE_TYPE)) )
                    throw new Exception("parameter 'type' for 'component' element is not specified or is unknown");
                
                component.setType(type);
                
                NodeList componentelements = componentNode.getChildNodes();
                
                for(int j = 0; j<componentelements.getLength(); j++)
                {
                    Node componentElementNode = componentelements.item(j);
                    
                    if(componentElementNode.getNodeType()== Node.ELEMENT_NODE)
                    {
                        // <component-class-name>
                        
                        if("component-class-name".equals(componentElementNode.getNodeName()))
                        {
                            component.setComponentClassName(getTextContent(componentElementNode));
                        }
                        
                        // <component-class-path>
                        
                        if("component-class-path".equals(componentElementNode.getNodeName()))
                        {
                            component.setComponentClassPath(getTextContent(componentElementNode));
                        }
                        
                        // <bootstrap-class-name>
                        
                        if("bootstrap-class-name".equals(componentElementNode.getNodeName()))
                        {
                            component.setBootstrapClassName(getTextContent(componentElementNode));
                        }
                        
                        // <bootstrap-class-path>
                        
                        if("bootstrap-class-path".equals(componentElementNode.getNodeName()))
                        {
                            component.setBootstrapClassPath(getTextContent(componentElementNode));
                        }
                    }
                }
                config.addComponentConfiguration(component);
            }
        }
        return config;
    }
    
    /**
     * Return the value of the attribute in the node
     * @param n the node
     * @param attName the name of the attribute
     * @return the value of the attribute, null if not found
     */
    private String getAttributeValue(Node n, String attName)
    {
        NamedNodeMap atts = n.getAttributes();
        
        Node att = atts.getNamedItem(attName);
        
        if(att != null)
        {
            return att.getNodeValue();
        }
        
        return null;
    }
    
    /**
     * Return the <i>text</i> element of a node
     * @param node
     * @return
     */
    private String getTextContent(Node node)
    {
         NodeList list = node.getChildNodes();
         
         for(int i= 0; i<list.getLength();i++)
         {
             if(list.item(i).getNodeType() == Node.TEXT_NODE )
             {
                 return list.item(i).getNodeValue();
             }
         }
         return null;
    }
    
    /**
     * Return the next sibling of a node,
     * regardless <i>text</i> node
     * @param node
     * @return
     */
    private Node getNextSibling(Node node)
    {
        Node result = node.getNextSibling();
        
        while(result.getNodeType() == Node.TEXT_NODE)
        {
            result = result.getNextSibling();
        }
        return result;
    }
    

    /**
     * Return the first child of a node,
     * regardless <i>text</i> node
     * @param node
     * @return
     */
    private Node getFirstChild(Node node)
    {
        Node result = node.getFirstChild();
        
        while(result.getNodeType() == Node.TEXT_NODE)
        {
            result = result.getNextSibling();
        }
        return result;
    }
    
    /**
     * Create a <code>SystemServiceConfig</code> object
     * from elements of a node representing a system service config
     * (a directory config, transporter config,...)
     * @param node
     * @return
     */
    private SystemServiceConfig createSystemServiceConfig(Node node)
    {
        SystemServiceConfig config = new SystemServiceConfig();
        
        Node className = getFirstChild(node);
        
        if("class-name".equals(className.getNodeName()))
        {
            config.setClassName(getTextContent(className));
        }
        
        Node parameters = getNextSibling(className);
        
        if(parameters != null && "parameters".equals(className.getNodeName()))
        {
            NamedNodeMap atts = parameters.getAttributes();
            
            HashMap params = new HashMap();
            
            for(int i = 0; i<atts.getLength(); i++)
            {
                params.put(atts.item(i).getNodeName(), atts.item(i).getNodeValue());
            }
            config.setParameters(params);
        }
        return config;
    }
}
