/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: LifeCycle.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.management.mbean;

import javax.jbi.JBIException;
import javax.management.AttributeChangeNotification;
import javax.management.ListenerNotFoundException;
import javax.management.MBeanNotificationInfo;
import javax.management.Notification;
import javax.management.NotificationBroadcaster;
import javax.management.NotificationBroadcasterSupport;
import javax.management.NotificationFilter;
import javax.management.NotificationListener;
import javax.management.ObjectName;

import org.objectweb.petals.component.LifeCycleElement;
import org.objectweb.petals.component.event.ElementStateListener;
import org.objectweb.petals.component.event.StateChangeFailedEvent;
import org.objectweb.petals.component.event.StateChangedEvent;


/**
 * Implementation of the <code>LifeCycle</code> mbean.
 * This lifecycle mbean sends notifications about changes of state.
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class LifeCycle implements LifeCycleMBean, NotificationBroadcaster, ElementStateListener
{
    protected LifeCycleElement element;
    
    private long sequenceNumber = 0;
    
    public LifeCycle(LifeCycleElement element)
    {
        this.element = element;
        
        element.setElementListener(this);
    }
    
    public ObjectName getMBeanName()
    {
        return element.getMBeanName();
    }
    
    /**
     * @see javax.jbi.management.LifeCycleMBean#getCurrentState()
     */
    public String getCurrentState()
    {
        return element.getCurrentState();
    }
    
    /**
     * As the shutDown action is performed in a separate thread,
     * no exception is thrown if a erro occured during the action.
     * A notification will be thrown in case of error.
     * @see javax.jbi.management.LifeCycleMBean#shutDown()
     * @throws JBIException the current state does not allow this action
     */
    public void shutDown() throws JBIException
    {
        element.shutDown();
    }
    
    /**
     * As the start action is performed in a separate thread,
     * no exception is thrown if a erro occured during the action.
     * A notification will be thrown in case of error.
     * @see javax.jbi.management.LifeCycleMBean#start()
     * @throws JBIException the current state does not allow this action
     */
    public void start() throws JBIException
    {
        element.start();
    }
    
    /**
     * As the stop action is performed in a separate thread,
     * no exception is thrown if a erro occured during the action.
     * A notification will be thrown in case of error.
     * @see javax.jbi.management.LifeCycleMBean#stop()
     * @throws JBIException the current state does not allow this action
     */
    public void stop() throws JBIException
    {
        element.stop();
    }
    
    
    //
    // Notifications support
    //
    
    NotificationBroadcasterSupport notifSupport = new NotificationBroadcasterSupport() ;
    
    public void addNotificationListener(NotificationListener arg0,
            NotificationFilter arg1, Object arg2)
    throws IllegalArgumentException
    {
        notifSupport.addNotificationListener(arg0, arg1, arg2);
    }
    
    
    public MBeanNotificationInfo[] getNotificationInfo()
    {
        String[] types = new String[] { 
                AttributeChangeNotification.ATTRIBUTE_CHANGE 
        }; 
        
        String name = AttributeChangeNotification.class.getName(); 
        
        String description = "Notification about a change of the lifeCycle state or a state change failure."; 
        
        MBeanNotificationInfo info = 
            new MBeanNotificationInfo(types, name, description); 
        
        return new MBeanNotificationInfo[] {info}; 
    } 
    
  
    
    public void removeNotificationListener(NotificationListener arg0)
    throws ListenerNotFoundException
    {
        notifSupport.removeNotificationListener(arg0);
    }
    
    //////////////////////////////////////////////
    // ElementListener implementation
    //////////////////////////////////////////////
    
    /**
     * <i>stateChanged</i>> is called when the state of an element (JBI Component or SystemService)
     * changes.
     * A JMX-notification is thrown.
     * @see petals.management.element.ElementStateListener#stateChanged(java.util.EventObject)
     */
    public synchronized void stateChanged(StateChangedEvent event)
    {
        Notification n = 
            new AttributeChangeNotification(
                    getMBeanName(),
                    sequenceNumber++,
                    System.currentTimeMillis(),
                    "State changed : "+event.getOldState().getName()+" -> "+event.getNewState().getName(),
                    "state",
                    "String",
                    event.getOldState().getName(),
                    event.getNewState().getName()); 
        
        notifSupport.sendNotification(n); 
    }
    
    /**
     * <i>stateChangeFailed</i>> is called when the change of state of an element (JBI Component or SystemService)
     * fails.
     * A JMX-notification throws the relative JBIException, .
     * @see petals.management.element.ElementStateListener#stateChanged(java.util.EventObject)
     */
    public synchronized void stateChangeFailed(StateChangeFailedEvent event)
    {
        Notification n = 
            new AttributeChangeNotification(
                    getMBeanName(),
                    sequenceNumber++,
                    System.currentTimeMillis(),
                    "State change failed : "+event.getException().getMessage(),
                    "exception",
                    "String",
                    null,
                    event.getException().toString()); 
        
        notifSupport.sendNotification(n); 
    }
    
}
