/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: IncomingMessageQueue.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.messaging;


import java.util.LinkedList;

import org.objectweb.petals.util.ESBLogger;


/**
 * Message queue attached to a <code>Component</code> <code>DeliveryChannel</code>.
 * Messages recevied by the <code>Transporter</code> are stored in this
 * queue until the component accepts messages.
 * TODO IncomingMessageQueue: stores messages physicly , or manage the recevied messages differently: the Transporter keeps all messages, and when an 'accept' method is called, ask the Transporter to return a message, with the component name as parameter of the request. The Transporter has to manage multi thread calls, and blocking calls. The transporter has to manager message storing.
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class IncomingMessageQueue{

	private LinkedList messagesQueue;
	
	public IncomingMessageQueue()
	{
		ESBLogger.logFine(this,"IncomingMessageQueue()<init>");
		
		messagesQueue = new LinkedList();
	}
	
	/**
	 * Push <i>msg</i> in the queue and notify pop requesters
	 * @param msg
	 */
	public void push(MessageExchangeImpl msg)  
	{
		ESBLogger.logFine(this,"push("+msg+")");
		
		synchronized (messagesQueue)
		{
			messagesQueue.addLast(msg);
			
			messagesQueue.notify();
		}
	}
	
	/**
	 * Poll the oldest message. 
	 * If there is no message, wait until a message is pushed.
	 * @return a message
	 */
	public MessageExchangeImpl poll()
	{
		ESBLogger.logFine(this,"poll()");
		
		synchronized (messagesQueue) 
		{
			if(messagesQueue.isEmpty())
			{
				try 
				{
					messagesQueue.wait();
				} 
				catch (InterruptedException e) 
				{
					ESBLogger.logSevere("poll()",e);
				}
			}
			return (MessageExchangeImpl)messagesQueue.removeFirst();
		}
	}
	
	/**
	 * Poll the oldest message. 
	 * If there is no message, wait until a message is pushed or timeoutMS exceeded
	 * @return a message, or <b>null</b> after timeoutMS exceeded
	 */
	public MessageExchangeImpl poll(long timeoutMS) throws IllegalArgumentException,InterruptedException
	{
		ESBLogger.logFine(this,"poll("+timeoutMS+")");
		
		synchronized (messagesQueue) 
		{
			if(messagesQueue.isEmpty())
			{
				messagesQueue.wait(timeoutMS);
			}
			if(! messagesQueue.isEmpty())
			{
				return (MessageExchangeImpl)messagesQueue.removeFirst();
			}
			return null;
		}
	}

	/**
	 * Poll the oldest message. 
	 * If there is no message, return <b>null<n>.
	 * @return a message, or <b>null</b> if there is no message.
	 */
	public MessageExchangeImpl pollNoWait()
	{
		ESBLogger.logFine(this,"pollNoWait()");
		
		synchronized (messagesQueue) 
		{
			if(! messagesQueue.isEmpty())
			{
				return (MessageExchangeImpl)messagesQueue.removeFirst();
			}
			return null;
		}
	}
}
