/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: AddressResolverImpl.java,v 1.1 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package com.ebmwebsourcing.jbi.systemservicesimpl;
import java.util.HashMap;

import javax.jbi.JBIException;

import org.objectweb.petals.component.context.ComponentContextImpl;
import org.objectweb.petals.description.ContainerDescription;
import org.objectweb.petals.directory.Directory;
import org.objectweb.petals.messaging.MessageExchangeImpl;
import org.objectweb.petals.routing.Address;
import org.objectweb.petals.routing.AddressResolver;
import org.objectweb.petals.routing.RoutingException;
import org.objectweb.petals.util.ESBLogger;



/**
 * The <code>AddressResolver</code> resolves implicit addresses such
 * as <code>interface</code> or <code>service</code> into an <code>endpoint</code>.
 * Rules are used to help the <code>AddressResolver</code> in the choice of
 * the <code>endpoint</code>, if more than one implement the implicit address.
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 *
 */
public class AddressResolverImpl implements AddressResolver
{
    
    protected ContainerDescription containerDesc;
    protected Directory directory;
    
    /**
     * Resolve the address if necessary.
     * Resolution consist in finding an endpoint satisfying
     * <code>interface</code> or <code>service</code>.
     * the source of the exchange and the exchange help
     * the <code>AddressResolver</code> to find the endpoint.
     * The <code>containerName</code> and <code>ComponentName</code>
     * properties are set.
     * If <code>containerName</code> and <code>ComponentName</code> 
     * already exist, do nothing.
     * @param address
     * @param source
     * @param exchange
     */
    public void resolveAddress(Address address,ComponentContextImpl source, MessageExchangeImpl exchange) throws RoutingException
    {
        ESBLogger.logFine(this,"resolveAddress("+address+","+source+","+exchange+")");
        
        if(address.getContainer() == null && address.getComponent() == null)
        {
            if(address.getEndpoint() == null)
            {
                if(address.getService() == null)
                {
                    if(address.getInterfaceName() == null)
                    {
                        throw new RoutingException("Address to resolve has no properties defined.");
                    }
                    else
                    {
                        // resolve interface
                        throw new NoSuchMethodError("interface support not yet implemented.");
                    }
                }
                else
                {
                    //resolve service
                    Address[] addresses = null;
                    
                    try
                    {
                        addresses = directory.getAddressesForService(address.getService());
                    }
                    catch (JBIException e)
                    {
                        throw new RoutingException(e);
                    }
                    if(addresses.length == 0)
                    {
                        throw new RoutingException("No endpoint satisfy address: "+address);
                    }
                    Address destination = electAddress(addresses);
                    
                    address.setEndpoint(destination.getEndpoint());
                    
                    address.setComponent(destination.getComponent());
                    
                    address.setContainer(destination.getContainer());
                }
            }
            else
            {
                // endpoint already defined, set associated containerName and componentName
                // if necessary
                Address destination = null;
                try
                {
                    destination = directory.getAddressForEndpoint(address.getEndpoint());
                }
                catch (JBIException e)
                {
                    throw new RoutingException(e);
                }
                address.setComponent(destination.getComponent());
                
                address.setContainer(destination.getContainer());
            }
        }
    }
   
    /**
     * Elects the address in the list of endpoints as the message recipient, with the
     * defined strategy.
     * TODO electAddress: for the moment, the rule is :- if a local endpoint exists, elect it, - otherwise, elect the first endpoint of the list
     * @param addresses
     * @return the elected recipient 
     */
    private Address electAddress(Address[] addresses)
    {
        ESBLogger.logFine(this,"electAddress("+addresses+")");
        
        String localContainerName = containerDesc.getName();
        
        for(int i = 0; i< addresses.length; i++)
        {
            if(localContainerName.equals(addresses[i].getContainer()))
            {
                return addresses[i];
            }
        }
        return addresses[0];
    }

    /**
     * do nothing
     * @see javax.jbi.component.ComponentLifeCycle#init(javax.jbi.component.ComponentContext)
     */
    public void init(HashMap arg0) throws JBIException{}

    /**
     * do nothing
     * @see javax.jbi.component.ComponentLifeCycle#shutDown()
     */
    public void shutDown() throws JBIException{}
 
    /**
     * do nothing
     * @see javax.jbi.component.ComponentLifeCycle#start()
     */
    public void start() throws JBIException{}
 
    /**
     * do nothing
     * @see javax.jbi.component.ComponentLifeCycle#stop()
     */
    public void stop() throws JBIException{}
  
    
    public void initContainerDescription(ContainerDescription containerDesc)
    {
        ESBLogger.logFine(this,"initContainerDescription("+containerDesc+")");
        
        this.containerDesc = containerDesc;
    }
    
    public void initDirectory(Directory directory)
    {
        ESBLogger.logFine(this,"initDirectory("+directory+")");
        
        this.directory = directory;
    }
}
